@@ -10,8 +10,9 @@ use Carp 'croak';
 use Exporter 'import';
 use Scalar::Util 'blessed';
 use Encode ();
+use B;
 
-our $VERSION = '0.56';
+our $VERSION = '0.57';
 our @EXPORT_OK = qw(decode_json encode_json false from_json j to_json true);
 
 # Literal names
@@ -265,16 +266,8 @@ sub _encode_value {
 
   # Number (bitwise operators change behavior based on the internal value type)
 
-  # "0" & $x will modify the flags on the "0" on perl < 5.14, so use a copy
-  my $zero = "0";
-  # "0" & $num -> 0. "0" & "" -> "". "0" & $string -> a character.
-  # this maintains the internal type but speeds up the xor below.
-  my $check = $zero & $value;
   return $value
-    if length $check
-    # 0 ^ itself          -> 0    (false)
-    # $character ^ itself -> "\0" (true)
-    && !($check ^ $check)
+    if B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK)
     # filter out "upgraded" strings whose numeric form doesn't strictly match
     && 0 + $value eq $value
     # filter out inf and nan
